package com.sunmi.idcarddemo;

import android.Manifest;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.os.IBinder;
import android.util.Log;
import android.view.View;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityCompat;
import androidx.databinding.DataBindingUtil;

import com.bumptech.glide.Glide;
import com.bumptech.glide.load.engine.DiskCacheStrategy;
import com.bumptech.glide.request.RequestOptions;
import com.sunmi.idcarddemo.databinding.ActivityMainBinding;
import com.sunmi.idcardservice.CardCallback;
import com.sunmi.idcardservice.IDCardInfo;
import com.sunmi.idcardservice.IDCardServiceAidl;

public class MainActivity extends AppCompatActivity {
    public static final String TAG = "darren-id-card";
    boolean hasPermission;
    private ActivityMainBinding mBinding;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        mBinding = DataBindingUtil.setContentView(this, R.layout.activity_main);
        bindIDCardService();
    }

    private void bindIDCardService() {
        Intent intent = new Intent();
        intent.setPackage("com.sunmi.idcardservice");
        intent.setAction("com.sunmi.idcard");
        bindService(intent, mConnection, Context.BIND_AUTO_CREATE);
    }


    private IDCardServiceAidl mService;
    private ServiceConnection mConnection = new ServiceConnection() {
        @Override
        public void onServiceConnected(ComponentName name, IBinder service) {
            mService = IDCardServiceAidl.Stub.asInterface(service);
            Log.e("id_card", " 服务已连接");
            Toast.makeText(MainActivity.this, "服务已连接", Toast.LENGTH_SHORT).show();
        }

        @Override
        public void onServiceDisconnected(ComponentName name) {
            mService = null;
            Log.e("id_card", "服务已断开");
            Toast.makeText(MainActivity.this, "服务已断开", Toast.LENGTH_SHORT).show();
        }
    };

    public void onMifare(View view) {
        startActivity(new Intent(this, MiFareActivity.class));
    }

    public void read(View view) {
        try {
            if (mService != null) {
                IDCardInfo cardInfo = mService.readCard();
                if (cardInfo != null) {
                    //显示图片
                    showInfo(cardInfo);
                    Log.i(TAG, "read: " + cardInfo.toString());
                } else {
                    Log.i(TAG, "card info is null ");
                }
            } else {
                Log.i(TAG, "service is null");
            }
        } catch (Exception e) {
            Log.i(TAG, "got error: " + e.getMessage());
            e.printStackTrace();
        }
    }

    public void auto(View view) {
        try {
            mService.readCardAuto(new CardCallback.Stub() {
                @Override
                public void getCardData(final IDCardInfo info, int code) {
                    Log.i(TAG, "getCardData: code=" + code + ", id info:" + info);
                    if (code == 10) {
                        //读取成功
                        Log.i(TAG, "getCardData: " + info.toString());


                        runOnUiThread(new Runnable() {
                            @Override
                            public void run() {
                                showInfo(info);
                            }
                        });

                    } else if (code == -10) {
                        //读取失败
                        Log.i(TAG, "auto read card did not get data");
                    } else {
                        Log.i(TAG, "what is wrong");
                    }
                }
            });
        } catch (Exception e) {
            Log.i(TAG, "auto read is error " + e.getMessage());
            e.printStackTrace();
        }
    }

    public void cancelAuto(View v) {
        try {
            mService.cancelAutoReading();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void showInfo(IDCardInfo info) {
        if (hasPermission) {
            Glide.with(this)
                    .load(info.getImageAddress())
                    .apply(new RequestOptions()
                            .skipMemoryCache(true)
                            .diskCacheStrategy(DiskCacheStrategy.NONE))
                    .into(mBinding.image);
        }
        // IdUid 取前16位，后四位为UID识读状态码：9000 标识身份证UID成功。
        String uid = info.getIdUid();
        mBinding.text.setText("uid: " + uid.substring(0, uid.length() - 4) + "\n"
                + "姓名: " + info.getName() + "\n"
                + "性别: " + info.getGender() + "\n"
                + "民族: " + info.getNation() + "\n"
                + "出生日期: " + info.getBirthDay() + "\n"
                + "地址: " + info.getAddress() + "\n"
                + "身份证号:" + info.getIdCard() + "\n"
                + "签发机关: " + info.getDepartment() + "\n"
                + "有效期限: " + info.getStartDate() + "-" + info.getEndDate() + "\n"
                /*+ "头像:" + info.getImageAddress() + "\n"*/
                + "指纹：" + info.getFirstFinger() + "\n" + info.getFirstFinger());
    }


    @Override
    protected void onStart() {
        super.onStart();
        hasPermission = ActivityCompat.checkSelfPermission(this, Manifest.permission.WRITE_EXTERNAL_STORAGE) == PackageManager.PERMISSION_GRANTED;
        if (!hasPermission) {
            ActivityCompat.requestPermissions(this, new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE}, 101);
        }
    }

    @Override
    protected void onStop() {
        super.onStop();
        if (mService != null) {
            try {
                mService.cancelAutoReading();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (mConnection != null) {
            try {
                unbindService(mConnection);
                Toast.makeText(MainActivity.this, "ID服务已断开", Toast.LENGTH_SHORT).show();
            } catch (Exception e) {
                e.printStackTrace();
            }
            mService = null;
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        if (requestCode == 101) {
            hasPermission = grantResults[0] == PackageManager.PERMISSION_GRANTED;
        }
    }
}
